import os
import io
import sys
import time
from machine import Pin
from time import sleep

log_filename = 'log.txt'
log_file = None

def format_msg(level, msg, ts=None):
    if ts is None:
        ts = time.localtime()
    return f'{ts[0]:04d}-{ts[1]:02d}-{ts[2]:02d}T{ts[3]:02d}:{ts[4]:02d}:{ts[5]:02d} {level:-8s} {msg}'

def log(level, msg):
    global log_file
    if log_file is None:
        log_file = open(log_filename, 'w')
    s = format_msg(level, msg)
    print(s)
    log_file.write(s)
    log_file.write('\n')

def debug(msg):
    log('debug', msg)

def info(msg):
    log('info', msg)

def warn(msg):
    log('warning', msg)

def error(msg):
    log('error', msg)

def exception(exc):
    with io.StringIO() as buf:
        sys.print_exception(exc, buf)
        buf.seek(0)
        for line in buf:
            log('error', line.rstrip())

def fatal(exc, blink=3, pin=None):
    exception(exc)
    if pin is None:
        machine = os.uname().machine
        if 'Pico W' in machine:
            pin = 'LED'
        elif 'Pico' in machine:
            pin = 25
        else:
            error(f'Cannot blink: unknown board {machine}')
    if pin is not None:
        led = Pin(pin, Pin.OUT)
        while True:
            for i in range(count):
                led.on()
                sleep(0.2)
                led.off()
                sleep(0.2)
            sleep(1)
